@@ -412,7 +412,7 @@ void KCoreDirListerCache::stop(KCoreDirLister *lister, bool silent)
     qCDebug(KIO_CORE_DIRLISTER) << "lister:" << lister << "silent=" << silent;
 
     const QList<QUrl> urls = lister->d->lstDirs;
-    Q_FOREACH (const QUrl &url, urls) {
+    for (const QUrl &url : urls) {
         stopListingUrl(lister, url, silent);
     }
 
@@ -1097,15 +1097,15 @@ void KCoreDirListerCache::handleDirDirty(const QUrl &url)
     // This also means we can forget about pending updates to individual files in that dir
     const QString dir = url.toLocalFile();
     QString dirPath = dir;
-    if (!dirPath.endsWith('/')) {
-        dirPath += '/';
+    if (!dirPath.endsWith(QLatin1Char('/'))) {
+        dirPath += QLatin1Char('/');
     }
     QMutableSetIterator<QString> pendingIt(pendingUpdates);
     while (pendingIt.hasNext()) {
         const QString updPath = pendingIt.next();
         qCDebug(KIO_CORE_DIRLISTER) << "had pending update" << updPath;
         if (updPath.startsWith(dirPath) &&
-                updPath.indexOf('/', dirPath.length()) == -1) { // direct child item
+                updPath.indexOf(QLatin1Char('/'), dirPath.length()) == -1) { // direct child item
             qCDebug(KIO_CORE_DIRLISTER) << "forgetting about individual update to" << updPath;
             pendingIt.remove();
         }
@@ -1571,19 +1571,20 @@ void KCoreDirListerCache::renameDir(const QUrl &oldUrl, const QUrl &newUrl)
                                                  newDirUrl.adjusted(QUrl::StripTrailingSlash),
                                                  dir));
             // Rename all items under that dir
-
+            // If all items of the directory change the same part of their url, the order is not
+            // changed, therefore just change it in the list.
             for (auto kit = dir->lstItems.begin(), kend = dir->lstItems.end(); kit != kend; ++kit) {
                 const KFileItem oldItem = *kit;
                 KFileItem newItem = oldItem;
-
                 const QUrl &oldItemUrl = oldItem.url();
                 QUrl newItemUrl(oldItemUrl);
                 newItemUrl.setPath(concatPaths(newDirUrl.path(), oldItemUrl.fileName()));
                 qCDebug(KIO_CORE_DIRLISTER) << "renaming" << oldItemUrl << "to" << newItemUrl;
                 newItem.setUrl(newItemUrl);
-                reinsert(newItem, oldItemUrl);
 
                 listers |= emitRefreshItem(oldItem, newItem);
+                // Change the item
+                kit->setUrl(newItemUrl);
             }
         }
     }
@@ -1668,7 +1669,7 @@ void KCoreDirListerCache::removeDirFromCache(const QUrl &dir)
 {
     qCDebug(KIO_CORE_DIRLISTER) << dir;
     const QList<QUrl> cachedDirs = itemsCached.keys(); // seems slow, but there's no qcache iterator...
-    foreach (const QUrl &cachedDir, cachedDirs) {
+    for (const QUrl &cachedDir : cachedDirs) {
         if (dir == cachedDir || dir.isParentOf(cachedDir)) {
             itemsCached.remove(cachedDir);
         }
@@ -1920,7 +1921,7 @@ void KCoreDirListerCache::itemsDeleted(const QList<KCoreDirLister *> &listers, c
         kdl->d->emitItemsDeleted(deletedItems);
     }
 
-    Q_FOREACH (const KFileItem &item, deletedItems) {
+    for (const KFileItem &item : deletedItems) {
         if (item.isDir()) {
             deleteDir(item.url());
         }
@@ -2047,7 +2048,7 @@ void KCoreDirListerCache::printDebug()
     for (; dit != directoryData.constEnd(); ++dit) {
         QString list;
         foreach (KCoreDirLister *listit, (*dit).listersCurrentlyListing) {
-            list += " 0x" + QString::number(reinterpret_cast<qlonglong>(listit), 16);
+            list += QLatin1String(" 0x") + QString::number(reinterpret_cast<qlonglong>(listit), 16);
         }
         qCDebug(KIO_CORE_DIRLISTER) << "  " << dit.key() << (*dit).listersCurrentlyListing.count() << "listers:" << list;
         foreach (KCoreDirLister *listit, (*dit).listersCurrentlyListing) {
@@ -2062,7 +2063,7 @@ void KCoreDirListerCache::printDebug()
 
         list.clear();
         foreach (KCoreDirLister *listit, (*dit).listersCurrentlyHolding) {
-            list += " 0x" + QString::number(reinterpret_cast<qlonglong>(listit), 16);
+            list += QLatin1String(" 0x") + QString::number(reinterpret_cast<qlonglong>(listit), 16);
         }
         qCDebug(KIO_CORE_DIRLISTER) << "  " << dit.key() << (*dit).listersCurrentlyHolding.count() << "holders:" << list;
     }
@@ -2075,7 +2076,7 @@ void KCoreDirListerCache::printDebug()
 
     qCDebug(KIO_CORE_DIRLISTER) << "Items in cache:";
     const QList<QUrl> cachedDirs = itemsCached.keys();
-    foreach (const QUrl &cachedDir, cachedDirs) {
+    for (const QUrl &cachedDir : cachedDirs) {
         DirItem *dirItem = itemsCached.object(cachedDir);
         qCDebug(KIO_CORE_DIRLISTER) << "   " << cachedDir << "rootItem:"
                  << (!dirItem->rootItem.isNull() ? dirItem->rootItem.url().toString() : QStringLiteral("NULL"))
@@ -2297,7 +2298,7 @@ void KCoreDirLister::setNameFilter(const QString &nameFilter)
     d->settings.lstFilters.clear();
     d->nameFilter = nameFilter;
     // Split on white space
-    const QStringList list = nameFilter.split(' ', QString::SkipEmptyParts);
+    const QStringList list = nameFilter.split(QLatin1Char(' '), QString::SkipEmptyParts);
     for (QStringList::const_iterator it = list.begin(); it != list.end(); ++it) {
         d->settings.lstFilters.append(QRegExp(*it, Qt::CaseInsensitive, QRegExp::Wildcard));
     }
@@ -2774,7 +2775,7 @@ KFileItem KCoreDirLister::cachedItemForUrl(const QUrl &url)
 
 QSet<QString> KCoreDirListerCache::filesInDotHiddenForDir(const QString& dir)
 {
-    const QString path = dir + "/.hidden";
+    const QString path = dir + QLatin1String("/.hidden");
     QFile dotHiddenFile(path);
 
     if (dotHiddenFile.exists()) {
