@@ -17,6 +17,10 @@
     Boston, MA 02110-1301, USA.
 
 */
+
+// TODO: remove me
+#undef QT_NO_CAST_FROM_ASCII
+
 #include "kfileplacesmodel.h"
 #include "kfileplacesitem_p.h"
 
@@ -26,10 +30,10 @@
 #endif
 
 #include <QCoreApplication>
-#include <QtCore/QDir>
-#include <QtCore/QMimeData>
-#include <QtCore/QTimer>
-#include <QtCore/QFile>
+#include <QDir>
+#include <QMimeData>
+#include <QTimer>
+#include <QFile>
 #include <QColor>
 #include <QAction>
 #include <QUrlQuery>
@@ -46,6 +50,7 @@
 
 #include <kio/job.h>
 #include <kprotocolinfo.h>
+#include <KCoreDirLister>
 
 #include <kconfig.h>
 #include <kconfiggroup.h>
@@ -76,6 +81,8 @@ namespace {
                 return QStringLiteral("GroupState-Devices-IsHidden");
         case KFilePlacesModel::RemovableDevicesType:
                 return QStringLiteral("GroupState-RemovableDevices-IsHidden");
+        case KFilePlacesModel::TagsType:
+                return QStringLiteral("GroupState-Tags-IsHidden");
         default:
             Q_UNREACHABLE();
         }
@@ -90,11 +97,11 @@ namespace {
 
     static QString timelineDateString(int year, int month, int day = 0)
     {
-        const QString dateFormat("%1-%2");
+        const QString dateFormat = QStringLiteral("%1-%2");
 
-        QString date = dateFormat.arg(year).arg(month, 2, 10, QChar('0'));
+        QString date = dateFormat.arg(year).arg(month, 2, 10, QLatin1Char('0'));
         if (day > 0) {
-            date += QString("-%1").arg(day, 2, 10, QChar('0'));
+            date += QStringLiteral("-%1").arg(day, 2, 10, QLatin1Char('0'));
         }
         return date;
     }
@@ -113,7 +120,7 @@ namespace {
             const int day = date.day();
 
             timelineUrl = QUrl(timelinePrefix + timelineDateString(year, month) +
-                  '/' + timelineDateString(year, month, day));
+                  QLatin1Char('/') + timelineDateString(year, month, day));
         } else if (path.endsWith(QLatin1String("/thismonth"))) {
             const QDate date = QDate::currentDate();
             timelineUrl = QUrl(timelinePrefix + timelineDateString(date.year(), date.month()));
@@ -128,41 +135,28 @@ namespace {
         return timelineUrl;
     }
 
-    static QUrl searchUrlForType(const QString &type)
-    {
-        const QString jsonQuery(QStringLiteral("{\"dayFilter\": 0,\
-                                                 \"monthFilter\": 0, \
-                                                 \"yearFilter\": 0, \
-                                                 \"type\": [ \"%1\"]}"));
-        QUrl url;
-        url.setScheme(QStringLiteral("baloosearch"));
-
-        QUrlQuery urlQuery;
-        urlQuery.addQueryItem(QStringLiteral("json"), jsonQuery.arg(type).simplified());
-        url.setQuery(urlQuery);
-
-        return url;
-    }
-
     static QUrl createSearchUrl(const QUrl &url)
     {
-        QUrl searchUrl;
+        QUrl searchUrl = url;
 
         const QString path = url.toDisplayString(QUrl::PreferLocalFile);
 
-        if (path.endsWith(QLatin1String("/documents"))) {
-            searchUrl = searchUrlForType(QStringLiteral("Document"));
-        } else if (path.endsWith(QLatin1String("/images"))) {
-            searchUrl = searchUrlForType(QStringLiteral("Image"));
-        } else if (path.endsWith(QLatin1String("/audio"))) {
-            searchUrl = searchUrlForType(QStringLiteral("Audio"));
-        } else if (path.endsWith(QLatin1String("/videos"))) {
-            searchUrl = searchUrlForType(QStringLiteral("Video"));
-        } else {
-            qWarning() << "Invalid search url:" << url;
-            searchUrl = url;
+        const QStringList validSearchPaths = {
+            QStringLiteral("/documents"),
+            QStringLiteral("/images"),
+            QStringLiteral("/audio"),
+            QStringLiteral("/videos")
+        };
+
+        for (const QString &validPath : validSearchPaths) {
+            if (path.endsWith(validPath)) {
+                searchUrl.setScheme(QStringLiteral("baloosearch"));
+                return searchUrl;
+            }
         }
 
+        qWarning() << "Invalid search url:" << url;
+
         return searchUrl;
     }
 }
@@ -170,11 +164,51 @@ namespace {
 class Q_DECL_HIDDEN KFilePlacesModel::Private
 {
 public:
-    Private(KFilePlacesModel *self)
+    explicit Private(KFilePlacesModel *self)
         : q(self),
           bookmarkManager(nullptr),
-          fileIndexingEnabled(isFileIndexingEnabled())
+          fileIndexingEnabled(isFileIndexingEnabled()),
+          tags(),
+          tagsLister(new KCoreDirLister())
     {
+        if (KProtocolInfo::isKnownProtocol(QStringLiteral("tags"))) {
+            connect(tagsLister, &KCoreDirLister::itemsAdded, q, [this](const QUrl&, const KFileItemList& items) {
+
+                if(tags.isEmpty()) {
+                    QList<QUrl> existingBookmarks;
+
+                    KBookmarkGroup root = bookmarkManager->root();
+                    KBookmark bookmark = root.first();
+
+                    while (!bookmark.isNull()) {
+                        existingBookmarks.append(bookmark.url());
+                        bookmark = root.next(bookmark);
+                    }
+
+                    if (!existingBookmarks.contains(QUrl(tagsUrlBase))) {
+                        KBookmark alltags = KFilePlacesItem::createSystemBookmark(bookmarkManager, QStringLiteral("All tags"), i18n("All tags"), QUrl(tagsUrlBase), QStringLiteral("tag"));
+                    }
+                }
+
+                for (const KFileItem &item: items) {
+                    const QString name = item.name();
+
+                     if (!tags.contains(name)) {
+                         tags.append(name);
+                    }
+                }
+                _k_reloadBookmarks();
+            });
+
+            connect(tagsLister, &KCoreDirLister::itemsDeleted, q, [this](const KFileItemList& items) {
+                for (const KFileItem &item: items) {
+                    tags.removeAll(item.name());
+                }
+                _k_reloadBookmarks();
+            });
+
+            tagsLister->openUrl(QUrl(tagsUrlBase), KCoreDirLister::OpenUrlFlag::Reload);
+        }
     }
 
     ~Private()
@@ -182,7 +216,7 @@ public:
         qDeleteAll(items);
     }
 
-    KFilePlacesModel *q;
+    KFilePlacesModel * const q;
 
     QList<KFilePlacesItem *> items;
     QVector<QString> availableDevices;
@@ -200,13 +234,17 @@ public:
     QList<KFilePlacesItem *> loadBookmarkList();
     int findNearestPosition(int source, int target);
 
+    QVector<QString> tags;
+    const QString tagsUrlBase = QStringLiteral("tags:/");
+    KCoreDirLister* tagsLister;
+
     void _k_initDeviceList();
     void _k_deviceAdded(const QString &udi);
     void _k_deviceRemoved(const QString &udi);
     void _k_itemChanged(const QString &udi);
     void _k_reloadBookmarks();
-    void _k_storageSetupDone(Solid::ErrorType error, QVariant errorData);
-    void _k_storageTeardownDone(Solid::ErrorType error, QVariant errorData);
+    void _k_storageSetupDone(Solid::ErrorType error, const QVariant &errorData);
+    void _k_storageTeardownDone(Solid::ErrorType error, const QVariant &errorData);
 
 private:
     bool isBalooUrl(const QUrl &url) const;
@@ -215,7 +253,7 @@ private:
 KFilePlacesModel::KFilePlacesModel(const QString &alternativeApplicationName, QObject *parent)
     : QAbstractItemModel(parent), d(new Private(this))
 {
-    const QString file = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + "/user-places.xbel";
+    const QString file = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation) + QLatin1String("/user-places.xbel");
     d->bookmarkManager = KBookmarkManager::managerForExternalFile(file);
     d->alternativeApplicationName = alternativeApplicationName;
 
@@ -239,13 +277,35 @@ KFilePlacesModel::KFilePlacesModel(const QString &alternativeApplicationName, QO
         KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
                                               QStringLiteral("Home"), I18N_NOOP2("KFile System Bookmarks", "Home"),
                                               QUrl::fromLocalFile(QDir::homePath()), QStringLiteral("user-home"));
+
+        // Some distros may not create various standard XDG folders by default
+        // so check for their existence before adding bookmarks for them
+        const QString desktopFolder = QStandardPaths::writableLocation(QStandardPaths::DesktopLocation);
+        if (QDir(desktopFolder).exists()) {
+            KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
+                                                QStringLiteral("Desktop"), I18N_NOOP2("KFile System Bookmarks", "Desktop"),
+                                                QUrl::fromLocalFile(desktopFolder), QStringLiteral("user-desktop"));
+        }
+        const QString documentsFolder = QStandardPaths::writableLocation(QStandardPaths::DocumentsLocation);
+        if (QDir(documentsFolder).exists()) {
+            KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
+                                                QStringLiteral("Documents"), I18N_NOOP2("KFile System Bookmarks", "Documents"),
+                                                QUrl::fromLocalFile(documentsFolder), QStringLiteral("folder-documents"));
+        }
+        const QString downloadFolder = QStandardPaths::writableLocation(QStandardPaths::DownloadLocation);
+        if (QDir(downloadFolder).exists()) {
+            KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
+                                                QStringLiteral("Downloads"), I18N_NOOP2("KFile System Bookmarks", "Downloads"),
+                                                QUrl::fromLocalFile(downloadFolder), QStringLiteral("folder-downloads"));
+        }
+
         KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
                                               QStringLiteral("Network"), I18N_NOOP2("KFile System Bookmarks", "Network"),
-                                              QUrl(QStringLiteral("remote:/")), QStringLiteral("network-workgroup"));
+                                              QUrl(QStringLiteral("remote:/")), QStringLiteral("folder-network"));
 #if defined(_WIN32_WCE)
         // adding drives
+        const QString driveIcon = QStringLiteral("drive-harddisk");
         foreach (const QFileInfo &info, QDir::drives()) {
-            QString driveIcon = "drive-harddisk";
             KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
                                                   info.absoluteFilePath(), info.absoluteFilePath(),
                                                   QUrl::fromLocalFile(info.absoluteFilePath()), driveIcon);
@@ -253,7 +313,7 @@ KFilePlacesModel::KFilePlacesModel(const QString &alternativeApplicationName, QO
 #elif !defined(Q_OS_WIN)
         KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
                                               QStringLiteral("Root"), I18N_NOOP2("KFile System Bookmarks", "Root"),
-                                              QUrl::fromLocalFile(QStringLiteral("/")), QStringLiteral("folder-red"));
+                                              QUrl::fromLocalFile(QStringLiteral("/")), QStringLiteral("folder-root"));
 #endif
         KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
                                               QStringLiteral("Trash"), I18N_NOOP2("KFile System Bookmarks", "Trash"),
@@ -263,6 +323,7 @@ KFilePlacesModel::KFilePlacesModel(const QString &alternativeApplicationName, QO
         setDefaultMetadataItemForGroup(RemoteType);
         setDefaultMetadataItemForGroup(DevicesType);
         setDefaultMetadataItemForGroup(RemovableDevicesType);
+        setDefaultMetadataItemForGroup(TagsType);
 
         // Force bookmarks to be saved. If on open/save dialog and the bookmarks are not saved, QFile::exists
         // will always return false, which opening/closing all the time the open/save dialog would case the
@@ -281,12 +342,6 @@ KFilePlacesModel::KFilePlacesModel(const QString &alternativeApplicationName, QO
         KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
                                               QStringLiteral("Yesterday"), I18N_NOOP2("KFile System Bookmarks", "Yesterday"),
                                               QUrl(QStringLiteral("timeline:/yesterday")),  QStringLiteral("view-calendar-day"));
-        KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                              QStringLiteral("This Month"), I18N_NOOP2("KFile System Bookmarks", "This Month"),
-                                              QUrl(QStringLiteral("timeline:/thismonth")),  QStringLiteral("view-calendar-month"));
-        KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
-                                              QStringLiteral("Last Month"), I18N_NOOP2("KFile System Bookmarks", "Last Month"),
-                                              QUrl(QStringLiteral("timeline:/lastmonth")),  QStringLiteral("view-calendar-month"));
         KFilePlacesItem::createSystemBookmark(d->bookmarkManager,
                                               QStringLiteral("Documents"), I18N_NOOP2("KFile System Bookmarks", "Documents"),
                                              QUrl(QStringLiteral("search:/documents")),  QStringLiteral("folder-text"));
@@ -315,8 +370,7 @@ KFilePlacesModel::KFilePlacesModel(const QString &alternativeApplicationName, QO
                       "StorageAccess.ignored == false ]"));
 
     if (KProtocolInfo::isKnownProtocol(QStringLiteral("mtp"))) {
-        predicate.prepend("[");
-        predicate.append(" OR PortableMediaPlayer.supportedProtocols == 'mtp']");
+        predicate = QLatin1Char('[') + predicate + QLatin1String(" OR PortableMediaPlayer.supportedProtocols == 'mtp']");
     }
 
     d->predicate = Solid::Predicate::fromString(predicate);
@@ -536,7 +590,8 @@ void KFilePlacesModel::Private::_k_initDeviceList()
 
     const QList<Solid::Device> &deviceList = Solid::Device::listFromQuery(predicate);
 
-    foreach (const Solid::Device &device, deviceList) {
+    availableDevices.reserve(deviceList.size());
+    for (const Solid::Device &device : deviceList) {
         availableDevices << device.udi();
     }
 
@@ -658,42 +713,60 @@ QList<KFilePlacesItem *> KFilePlacesModel::Private::loadBookmarkList()
     KBookmarkGroup root = bookmarkManager->root();
     KBookmark bookmark = root.first();
     QVector<QString> devices = availableDevices;
+    QVector<QString> tagsList = tags;
 
     while (!bookmark.isNull()) {
-        QString udi = bookmark.metaDataItem(QStringLiteral("UDI"));
-        QString appName = bookmark.metaDataItem(QStringLiteral("OnlyInApp"));
-        QUrl url = bookmark.url();
-        auto it = std::find(devices.begin(), devices.end(), udi);
-        bool deviceAvailable = (it != devices.end());
-        if (it != devices.end()) {
-            devices.erase(it);
-        }
+        const QString udi = bookmark.metaDataItem(QStringLiteral("UDI"));
+        const QUrl url = bookmark.url();
+        const QString tag = bookmark.metaDataItem(QStringLiteral("tag"));
+        if (!udi.isEmpty() || url.isValid()) {
+            QString appName = bookmark.metaDataItem(QStringLiteral("OnlyInApp"));
 
-        bool allowedHere = appName.isEmpty() ||
-                ((appName == QCoreApplication::instance()->applicationName()) ||
-                 (appName == alternativeApplicationName));
-        bool isSupportedUrl = isBalooUrl(url) ? fileIndexingEnabled : true;
-        bool isSupportedScheme = supportedSchemes.isEmpty() || supportedSchemes.contains(url.scheme());
+            // If it's not a tag it's a device
+            if (tag.isEmpty()) {
+                auto it = std::find(devices.begin(), devices.end(), udi);
+                bool deviceAvailable = (it != devices.end());
+                if (deviceAvailable) {
+                    devices.erase(it);
+                }
 
-        if (isSupportedScheme && ((isSupportedUrl && udi.isEmpty() && allowedHere) || deviceAvailable)) {
+                bool allowedHere = appName.isEmpty() ||
+                        ((appName == QCoreApplication::instance()->applicationName()) ||
+                        (appName == alternativeApplicationName));
+                bool isSupportedUrl = isBalooUrl(url) ? fileIndexingEnabled : true;
+                bool isSupportedScheme = supportedSchemes.isEmpty() || supportedSchemes.contains(url.scheme());
 
-            KFilePlacesItem *item;
-            if (deviceAvailable) {
-                item = new KFilePlacesItem(bookmarkManager, bookmark.address(), udi);
-                // TODO: Update bookmark internal element
+                if (isSupportedScheme && ((isSupportedUrl && udi.isEmpty() && allowedHere) || deviceAvailable)) {
+
+                    KFilePlacesItem *item;
+                    if (deviceAvailable) {
+                        item = new KFilePlacesItem(bookmarkManager, bookmark.address(), udi);
+                        // TODO: Update bookmark internal element
+                    } else {
+                        item = new KFilePlacesItem(bookmarkManager, bookmark.address());
+                    }
+                    connect(item, SIGNAL(itemChanged(QString)),
+                            q, SLOT(_k_itemChanged(QString)));
+
+                    items << item;
+                }
             } else {
-                item = new KFilePlacesItem(bookmarkManager, bookmark.address());
+                auto it = std::find(tagsList.begin(), tagsList.end(), tag);
+                if (it != tagsList.end()) {
+                    tagsList.removeAll(tag);
+                    KFilePlacesItem *item = new KFilePlacesItem(bookmarkManager, bookmark.address());
+                    items << item;
+                    connect(item, SIGNAL(itemChanged(QString)),
+                                q, SLOT(_k_itemChanged(QString)));
+                }
             }
-            connect(item, SIGNAL(itemChanged(QString)),
-                    q, SLOT(_k_itemChanged(QString)));
-            items << item;
         }
 
         bookmark = root.next(bookmark);
     }
 
     // Add bookmarks for the remaining devices, they were previously unknown
-    foreach (const QString &udi, devices) {
+    for (const QString &udi : qAsConst(devices)) {
         bookmark = KFilePlacesItem::createDeviceBookmark(bookmarkManager, udi);
         if (!bookmark.isNull()) {
             KFilePlacesItem *item = new KFilePlacesItem(bookmarkManager,
@@ -705,8 +778,19 @@ QList<KFilePlacesItem *> KFilePlacesModel::Private::loadBookmarkList()
         }
     }
 
+    for (const QString& tag: tagsList) {
+        bookmark = KFilePlacesItem::createTagBookmark(bookmarkManager, tag);
+        if (!bookmark.isNull()) {
+            KFilePlacesItem *item = new KFilePlacesItem(bookmarkManager,
+                    bookmark.address(), tag);
+            connect(item, SIGNAL(itemChanged(QString)),
+                    q, SLOT(_k_itemChanged(QString)));
+            items << item;
+        }
+    }
+
     // return a sorted list based on groups
-    qStableSort(items.begin(), items.end(),
+    std::stable_sort(items.begin(), items.end(),
                 [](KFilePlacesItem *itemA, KFilePlacesItem *itemB) {
        return (itemA->groupType() < itemB->groupType());
     });
@@ -795,7 +879,7 @@ QMimeData *KFilePlacesModel::mimeData(const QModelIndexList &indexes) const
 
     QDataStream stream(&itemData, QIODevice::WriteOnly);
 
-    foreach (const QModelIndex &index, indexes) {
+    for (const QModelIndex &index : qAsConst(indexes)) {
         QUrl itemUrl = url(index);
         if (itemUrl.isValid()) {
             urls << itemUrl;
@@ -869,7 +953,7 @@ bool KFilePlacesModel::dropMimeData(const QMimeData *data, Qt::DropAction action
 
         KBookmarkGroup group = d->bookmarkManager->root();
 
-        foreach (const QUrl &url, urls) {
+        for (const QUrl &url : urls) {
             // TODO: use KIO::stat in order to get the UDS_DISPLAY_NAME too
             KIO::MimetypeJob *job = KIO::mimetype(url);
 
@@ -1052,7 +1136,7 @@ void KFilePlacesModel::setGroupHidden(const GroupType type, bool hidden)
     if (isGroupHidden(type) == hidden)
         return;
 
-    d->bookmarkManager->root().setMetaDataItem(stateNameForGroupType(type), (hidden ? QStringLiteral("true") : QStringLiteral("false")));    
+    d->bookmarkManager->root().setMetaDataItem(stateNameForGroupType(type), (hidden ? QStringLiteral("true") : QStringLiteral("false")));
     d->reloadAndSignal();
     emit groupHiddenChanged(type, hidden);
 }
@@ -1148,7 +1232,7 @@ QAction *KFilePlacesModel::teardownActionForIndex(const QModelIndex &index) cons
 
         QString iconName;
         QString text;
-        QString label = data(index, Qt::DisplayRole).toString().replace('&', QLatin1String("&&"));
+        QString label = data(index, Qt::DisplayRole).toString().replace(QLatin1Char('&'), QLatin1String("&&"));
 
         if (device.is<Solid::OpticalDisc>()) {
             text = i18n("&Release '%1'", label);
@@ -1176,7 +1260,7 @@ QAction *KFilePlacesModel::ejectActionForIndex(const QModelIndex &index) const
 
     if (device.is<Solid::OpticalDisc>()) {
 
-        QString label = data(index, Qt::DisplayRole).toString().replace('&', QLatin1String("&&"));
+        QString label = data(index, Qt::DisplayRole).toString().replace(QLatin1Char('&'), QLatin1String("&&"));
         QString text = i18n("&Eject '%1'", label);
 
         return new QAction(QIcon::fromTheme(QStringLiteral("media-eject")), text, nullptr);
@@ -1210,7 +1294,7 @@ void KFilePlacesModel::requestEject(const QModelIndex &index)
 
         drive->eject();
     } else {
-        QString label = data(index, Qt::DisplayRole).toString().replace('&', QLatin1String("&&"));
+        QString label = data(index, Qt::DisplayRole).toString().replace(QLatin1Char('&'), QLatin1String("&&"));
         QString message = i18n("The device '%1' is not a disk and cannot be ejected.", label);
         emit errorMessage(message);
     }
@@ -1235,7 +1319,7 @@ void KFilePlacesModel::requestSetup(const QModelIndex &index)
     }
 }
 
-void KFilePlacesModel::Private::_k_storageSetupDone(Solid::ErrorType error, QVariant errorData)
+void KFilePlacesModel::Private::_k_storageSetupDone(Solid::ErrorType error, const QVariant &errorData)
 {
     QPersistentModelIndex index = setupInProgress.take(q->sender());
 
@@ -1259,7 +1343,7 @@ void KFilePlacesModel::Private::_k_storageSetupDone(Solid::ErrorType error, QVar
 
 }
 
-void KFilePlacesModel::Private::_k_storageTeardownDone(Solid::ErrorType error, QVariant errorData)
+void KFilePlacesModel::Private::_k_storageTeardownDone(Solid::ErrorType error, const QVariant &errorData)
 {
     if (error && errorData.isValid()) {
         emit q->errorMessage(errorData.toString());
