@@ -27,6 +27,7 @@
 #include "kurlnavigatordropdownbutton_p.h"
 #include "kurlnavigatorbutton_p.h"
 #include "kurlnavigatortogglebutton_p.h"
+#include "kurlnavigatorpathselectoreventfilter_p.h"
 #include "urlutil_p.h"
 
 #include <kfileitem.h>
@@ -37,9 +38,9 @@
 #include <kurlcompletion.h>
 #include <kurifilter.h>
 
-#include <QtCore/QDir>
-#include <QtCore/QLinkedList>
-#include <QtCore/QTimer>
+#include <QDir>
+#include <QLinkedList>
+#include <QTimer>
 #include <QApplication>
 #include <QBoxLayout>
 #include <QClipboard>
@@ -104,9 +105,9 @@ public:
      * of the navigator if the left mouse button has been used. If the middle
      * mouse button has been used, the signal tabRequested() will be emitted.
      */
-    void slotNavigatorButtonClicked(const QUrl &url, Qt::MouseButton button);
+    void slotNavigatorButtonClicked(const QUrl &url, Qt::MouseButton button, Qt::KeyboardModifiers modifiers);
 
-    void openContextMenu();
+    void openContextMenu(const QPoint &p);
 
     void slotPathBoxChanged(const QString &text);
 
@@ -191,7 +192,7 @@ public:
     QUrl m_homeUrl;
     QStringList m_customProtocols;
     QWidget *m_dropWidget;
-    KUrlNavigator *q;
+    KUrlNavigator * const q;
 };
 
 KUrlNavigator::Private::Private(KUrlNavigator *q, KFilePlacesModel *placesModel) :
@@ -213,15 +214,15 @@ KUrlNavigator::Private::Private(KUrlNavigator *q, KFilePlacesModel *placesModel)
     q(q)
 {
     m_layout->setSpacing(0);
-    m_layout->setMargin(0);
+    m_layout->setContentsMargins(0, 0, 0, 0);
 
     // initialize the places selector
     q->setAutoFillBackground(false);
 
     if (placesModel != nullptr) {
         m_placesSelector = new KUrlNavigatorPlacesSelector(q, placesModel);
-        connect(m_placesSelector, SIGNAL(placeActivated(QUrl)),
-                q, SLOT(setLocationUrl(QUrl)));
+        connect(m_placesSelector, &KUrlNavigatorPlacesSelector::placeActivated,
+                q, &KUrlNavigator::setLocationUrl);
         connect(m_placesSelector, &KUrlNavigatorPlacesSelector::tabRequested,
                 q, &KUrlNavigator::tabRequested);
 
@@ -256,8 +257,8 @@ KUrlNavigator::Private::Private(KUrlNavigator *q, KFilePlacesModel *placesModel)
 
     connect(m_pathBox, SIGNAL(returnPressed()),
             q, SLOT(slotReturnPressed()));
-    connect(m_pathBox, SIGNAL(urlActivated(QUrl)),
-            q, SLOT(setLocationUrl(QUrl)));
+    connect(m_pathBox, &KUrlComboBox::urlActivated,
+            q, &KUrlNavigator::setLocationUrl);
     connect(m_pathBox, SIGNAL(editTextChanged(QString)),
             q, SLOT(slotPathBoxChanged(QString)));
 
@@ -279,7 +280,7 @@ KUrlNavigator::Private::Private(KUrlNavigator *q, KFilePlacesModel *placesModel)
 
     q->setContextMenuPolicy(Qt::CustomContextMenu);
     connect(q, SIGNAL(customContextMenuRequested(QPoint)),
-            q, SLOT(openContextMenu()));
+            q, SLOT(openContextMenu(QPoint)));
 }
 
 void KUrlNavigator::Private::initialize(const QUrl &url)
@@ -351,7 +352,7 @@ void KUrlNavigator::Private::slotProtocolChanged(const QString &protocol)
     } else {
         // With no authority set we'll get e.g. "ftp:" instead of "ftp://".
         // We want the latter, so let's set an empty authority.
-        url.setAuthority(QStringLiteral(""));
+        url.setAuthority(QString());
     }
 
     m_pathBox->setEditUrl(url);
@@ -367,6 +368,11 @@ void KUrlNavigator::Private::openPathSelectorMenu()
 
     QString spacer;
     QPointer<QMenu> popup = new QMenu(q);
+
+    auto *popupFilter = new KUrlNavigatorPathSelectorEventFilter(popup.data());
+    connect(popupFilter, &KUrlNavigatorPathSelectorEventFilter::tabRequested, q, &KUrlNavigator::tabRequested);
+    popup->installEventFilter(popupFilter);
+
     popup->setLayoutDirection(Qt::LeftToRight);
 
     const QUrl placeUrl = retrievePlaceUrl();
@@ -394,8 +400,8 @@ void KUrlNavigator::Private::openPathSelectorMenu()
         popup->addAction(action);
 
         ++idx;
-        spacer.append("  ");
-        dirName = path.section('/', idx, idx);
+        spacer.append(QLatin1String("  "));
+        dirName = path.section(QLatin1Char('/'), idx, idx);
     } while (!dirName.isEmpty());
 
     const QPoint pos = q->mapToGlobal(m_dropDownButton->geometry().bottomRight());
@@ -430,7 +436,7 @@ void KUrlNavigator::Private::switchView()
         m_pathBox->setFocus();
     }
 
-    emit q->requestActivation();
+    q->requestActivation();
     emit q->editableStateChanged(m_editable);
 }
 
@@ -442,16 +448,16 @@ void KUrlNavigator::Private::dropUrls(const QUrl &destination, QDropEvent *event
     }
 }
 
-void KUrlNavigator::Private::slotNavigatorButtonClicked(const QUrl &url, Qt::MouseButton button)
+void KUrlNavigator::Private::slotNavigatorButtonClicked(const QUrl &url, Qt::MouseButton button, Qt::KeyboardModifiers modifiers)
 {
-    if (button & Qt::LeftButton) {
-        q->setLocationUrl(url);
-    } else if (button & Qt::MidButton) {
+    if (button & Qt::MidButton || (button & Qt::LeftButton && modifiers & Qt::ControlModifier)) {
         emit q->tabRequested(url);
+    } else if (button & Qt::LeftButton) {
+        q->setLocationUrl(url);
     }
 }
 
-void KUrlNavigator::Private::openContextMenu()
+void KUrlNavigator::Private::openContextMenu(const QPoint &p)
 {
     q->setActive(true);
 
@@ -469,6 +475,18 @@ void KUrlNavigator::Private::openContextMenu()
 
     popup->addSeparator();
 
+    //We are checking for receivers because it's odd to have a tab entry even if it's not supported, like in the case of the open dialog
+    if (q->receivers(SIGNAL(tabRequested(QUrl))) > 0) {
+        for (auto button : qAsConst(m_navButtons)) {
+            if (button->geometry().contains(p)) {
+                const auto url = button->url();
+                QAction* openInTab = popup->addAction(QIcon::fromTheme(QStringLiteral("tab-new")), i18n("Open %1 in tab", button->text()));
+                q->connect(openInTab, &QAction::triggered, q, [this, url](){ Q_EMIT q->tabRequested(url); });
+                break;
+            }
+        }
+    }
+
     // provide radiobuttons for toggling between the edit and the navigation mode
     QAction *editAction = popup->addAction(i18n("Edit"));
     editAction->setCheckable(true);
@@ -518,7 +536,9 @@ void KUrlNavigator::Private::slotPathBoxChanged(const QString &text)
     if (text.isEmpty()) {
         const QString protocol = q->locationUrl().scheme();
         m_protocols->setProtocol(protocol);
-        m_protocols->show();
+        if (m_customProtocols.count() != 1) {
+            m_protocols->show();
+        }
     } else {
         m_protocols->hide();
     }
@@ -562,7 +582,7 @@ void KUrlNavigator::Private::updateContent()
         QString placePath = placeUrl.path();
         removeTrailingSlash(placePath);
 
-        const int startIndex = placePath.count('/');
+        const int startIndex = placePath.count(QLatin1Char('/'));
         updateButtons(startIndex);
     }
 }
@@ -594,8 +614,8 @@ void KUrlNavigator::Private::updateButtons(int startIndex)
                 button->setForegroundRole(QPalette::WindowText);
                 connect(button, SIGNAL(urlsDropped(QUrl,QDropEvent*)),
                         q, SLOT(dropUrls(QUrl,QDropEvent*)));
-                connect(button, SIGNAL(clicked(QUrl,Qt::MouseButton)),
-                        q, SLOT(slotNavigatorButtonClicked(QUrl,Qt::MouseButton)));
+                connect(button, SIGNAL(clicked(QUrl,Qt::MouseButton,Qt::KeyboardModifiers)),
+                        q, SLOT(slotNavigatorButtonClicked(QUrl,Qt::MouseButton,Qt::KeyboardModifiers)));
                 connect(button, SIGNAL(finishedTextResolving()),
                         q, SLOT(updateButtonVisibility()));
                 appendWidget(button);
@@ -768,7 +788,7 @@ QUrl KUrlNavigator::Private::buttonUrl(int index) const
             path = QStringLiteral("/");
 #endif
         } else {
-            path = path.section('/', 0, index);
+            path = path.section(QLatin1Char('/'), 0, index);
         }
     }
 
@@ -816,7 +836,7 @@ bool KUrlNavigator::Private::isCompressedPath(const QUrl &url) const
 void KUrlNavigator::Private::removeTrailingSlash(QString &url) const
 {
     const int length = url.length();
-    if ((length > 0) && (url.at(length - 1) == QChar('/'))) {
+    if ((length > 0) && (url.at(length - 1) == QLatin1Char('/'))) {
         url.remove(length - 1, 1);
     }
 }
@@ -1010,7 +1030,7 @@ QUrl KUrlNavigator::uncommittedUrl() const
 {
     KUriFilterData filteredData(d->m_pathBox->currentText().trimmed());
     filteredData.setCheckForExecutables(false);
-    if (KUriFilter::self()->filterUri(filteredData, QStringList() << QStringLiteral("kshorturifilter") << QStringLiteral("kurisearchfilter"))) {
+    if (KUriFilter::self()->filterUri(filteredData, QStringList{ QStringLiteral("kshorturifilter"), QStringLiteral("kurisearchfilter")})) {
         return filteredData.uri();
     } else {
         return QUrl::fromUserInput(filteredData.typedString());
@@ -1027,11 +1047,11 @@ void KUrlNavigator::setLocationUrl(const QUrl &newUrl)
 
     // This will be used below; we define it here because in the lower part of the
     // code locationUrl() and url become the same URLs
-    const QUrl firstChildUrl = KIO::UrlUtil::firstChildUrl(locationUrl(), url);
+    QUrl firstChildUrl = KIO::UrlUtil::firstChildUrl(locationUrl(), url);
 
-    if ((url.scheme() == QLatin1String("tar")) || (url.scheme() == QLatin1String("zip"))) {
-        // The URL represents a tar- or zip-file. Check whether
-        // the URL is really part of the tar- or zip-file, otherwise
+    if ((url.scheme() == QLatin1String("tar")) || (url.scheme() == QLatin1String("zip")) || (url.scheme() == QLatin1String("krarc"))) {
+        // The URL represents a tar- or zip-file, or an archive file supported by krarc.
+        // Check whether the URL is really part of the archive file, otherwise
         // replace it by the local path again.
         bool insideCompressedPath = d->isCompressedPath(url);
         if (!insideCompressedPath) {
@@ -1047,9 +1067,10 @@ void KUrlNavigator::setLocationUrl(const QUrl &newUrl)
             }
         }
         if (!insideCompressedPath) {
-            // drop the tar: or zip: protocol since we are not
+            // drop the tar: or zip: or krarc: protocol since we are not
             // inside the compressed path
             url.setScheme(QStringLiteral("file"));
+            firstChildUrl.setScheme(QStringLiteral("file"));
         }
     }
 
@@ -1152,6 +1173,14 @@ void KUrlNavigator::keyReleaseEvent(QKeyEvent *event)
     QWidget::keyReleaseEvent(event);
 }
 
+void KUrlNavigator::mousePressEvent(QMouseEvent *event)
+{
+    if (event->button() == Qt::MiddleButton) {
+        requestActivation();
+    }
+    QWidget::mousePressEvent(event);
+}
+
 void KUrlNavigator::mouseReleaseEvent(QMouseEvent *event)
 {
     if (event->button() == Qt::MidButton) {
