@@ -40,8 +40,8 @@
 #include <kio/authinfo.h>
 #include <kntlm.h>
 
-#include <QtCore/QTextCodec>
-#include <QtCore/QCryptographicHash>
+#include <QTextCodec>
+#include <QCryptographicHash>
 
 Q_LOGGING_CATEGORY(KIO_HTTP_AUTH, "kf5.kio.kio_http.auth")
 
@@ -93,7 +93,7 @@ static QList<QByteArray> parseChallenge(QByteArray &ba, QByteArray *scheme, QByt
 
     // drop empty stuff from the given string, it would have to be skipped over and over again
     if (start != 0) {
-        ba = ba.mid(start);
+        ba.remove(0, start);
         end -= start;
         len -= start;
         start = 0;
@@ -229,7 +229,7 @@ QByteArray KAbstractHttpAuthentication::bestOffer(const QList<QByteArray> &offer
     QByteArray digestOffer;
     QByteArray ntlmOffer;
     QByteArray basicOffer;
-    Q_FOREACH (const QByteArray &offer, offers) {
+    for (const QByteArray &offer : offers) {
         const QByteArray scheme = offer.mid(0, offer.indexOf(' ')).toLower();
 #if HAVE_LIBGSSAPI
         if (scheme == "negotiate") { // krazy:exclude=strings
@@ -282,7 +282,7 @@ QList< QByteArray > KAbstractHttpAuthentication::splitOffers(const QList< QByteA
 {
     // first detect if one entry may contain multiple offers
     QList<QByteArray> alloffers;
-    foreach (QByteArray offer, offers) {
+    for (QByteArray offer : offers) {
         QByteArray scheme, cont;
 
         parseChallenge(offer, &scheme, &cont);
@@ -461,7 +461,7 @@ static QByteArray calculateResponse(const DigestAuthInfo &info, const QUrl &reso
 
     // qDebug() << "A1 => " << HA1;
 
-    // Calcualte H(A2)
+    // Calculate H(A2)
     authStr = info.method;
     authStr += ':';
     authStr += resource.path(QUrl::FullyEncoded).toLatin1();
@@ -480,7 +480,7 @@ static QByteArray calculateResponse(const DigestAuthInfo &info, const QUrl &reso
 
     // qDebug() << "A2 => " << HA2;
 
-    // Calcualte the response.
+    // Calculate the response.
     authStr = HA1;
     authStr += ':';
     authStr += info.nonce;
@@ -546,7 +546,7 @@ void KHttpDigestAuthentication::generateResponse(const QString &user, const QStr
     }
 
     Q_FOREACH (const QByteArray &path, valueForKey(m_challenge, "domain").split(' ')) {
-        QUrl u = m_resource.resolved(QUrl(path));
+        QUrl u = m_resource.resolved(QUrl(QString::fromUtf8(path)));
         if (u.isValid()) {
             info.digestURIs.append(u);
         }
@@ -776,9 +776,7 @@ void KHttpNtlmAuthentication::generateResponse(const QString &_user, const QStri
         }
     }
 
-    m_headerFragment = "NTLM ";
-    m_headerFragment += buf.toBase64();
-    m_headerFragment += "\r\n";
+    m_headerFragment = "NTLM " + buf.toBase64() + "\r\n";
 
     return;
 }
@@ -825,7 +823,7 @@ void KHttpNegotiateAuthentication::fillKioAuthInfo(KIO::AuthInfo *ai) const
 {
     authInfoBoilerplate(ai);
     //### does GSSAPI supply anything realm-like? dummy value for now.
-    ai->realmValue = QLatin1String("Negotiate");
+    ai->realmValue = QStringLiteral("Negotiate");
 }
 
 void KHttpNegotiateAuthentication::generateResponse(const QString &user, const QString &password)
